package org.apache.bcel.verifier;

import org.apache.bcel.Repository;
import org.apache.bcel.classfile.JavaClass;
import org.apache.bcel.classfile.Utility;
import org.apache.bcel.verifier.VerificationResult;
import org.apache.bcel.verifier.Verifier;
import org.apache.bcel.verifier.VerifierFactory;
import org.apache.bcel.verifier.VerifierFactoryObserver;

public class TransitiveHull implements VerifierFactoryObserver {

   private int indent;


   public static void main(String[] args) {
      if(args.length != 1) {
         System.out.println("Need exactly one argument: The root class to verify.");
         System.exit(1);
      }

      int dotclasspos = args[0].lastIndexOf(".class");
      if(dotclasspos != -1) {
         args[0] = args[0].substring(0, dotclasspos);
      }

      args[0] = Utility.pathToPackage(args[0]);
      TransitiveHull th = new TransitiveHull();
      VerifierFactory.attach(th);
      VerifierFactory.getVerifier(args[0]);
      VerifierFactory.detach(th);
   }

   public void update(String className) {
      System.gc();

      for(int v = 0; v < this.indent; ++v) {
         System.out.print(" ");
      }

      System.out.println(className);
      ++this.indent;
      Verifier var7 = VerifierFactory.getVerifier(className);
      VerificationResult vr = var7.doPass1();
      if(vr != VerificationResult.VR_OK) {
         System.out.println("Pass 1:\n" + vr);
      }

      vr = var7.doPass2();
      if(vr != VerificationResult.VR_OK) {
         System.out.println("Pass 2:\n" + vr);
      }

      if(vr == VerificationResult.VR_OK) {
         try {
            JavaClass e = Repository.lookupClass(var7.getClassName());

            for(int i = 0; i < e.getMethods().length; ++i) {
               vr = var7.doPass3a(i);
               if(vr != VerificationResult.VR_OK) {
                  System.out.println(var7.getClassName() + ", Pass 3a, method " + i + " [\'" + e.getMethods()[i] + "\']:\n" + vr);
               }

               vr = var7.doPass3b(i);
               if(vr != VerificationResult.VR_OK) {
                  System.out.println(var7.getClassName() + ", Pass 3b, method " + i + " [\'" + e.getMethods()[i] + "\']:\n" + vr);
               }
            }
         } catch (ClassNotFoundException var6) {
            System.err.println("Could not find class " + var7.getClassName() + " in Repository");
         }
      }

      --this.indent;
   }
}
